% This MATLAB file computes an approximate solution for Proudman-Johnson eq.:
%  --------------------------------------------------------------------------  
%      u^(4)=f(u)=:R(u*u^(3)-u^(1)*u^(2))+sin(k*x) in (0,Pi)
%      u=u^(2)=0 on boundary of (0,Pi)
%  --------------------------------------------------------------------------  
% by Fourier-Galarkin method with usual floating point arithmetic.
  tic
% parameter input
  format long;
  input_data
  e = 10^(-15); % stopping criterion
  maxit = 100;  % maximum iteration number
  tol = eps^2;  % tolelance of approximate solution

  u = initial_value(N);      % set initial value
  D = pi/2*diag(1:N)^4;      % diagonal matrix D
  D = full(D);

  it = 1; err = realmax;
  formatSpec = 'R = %-6d N = %-6d k = %-2d\n';
  fprintf(formatSpec,R,N,k)
  fprintf('%5s %15s\r','its','relative error');
% Newton-Raphson iteration start
  while err > e 
    f = function_f(N,R,k,u);
    g = D*u - pi/2*f(1:N);

    G = D;             % Jacobian matrix G
    for j = 1:N
      df = function_df(N,R,j,u);
      G(:,j) = G(:,j) - pi/2*df(1:N);
    end 
   
    x = u - G\g;              % solve linear system
    x(abs(x)<tol) = 0;        % cut small values
    if norm(x,inf) < eps      % check some differences
      error('obtain trivial zero solution numerically')
      u = x;
    end
    error1 = norm(x-u,inf)/norm(u,inf);
    error2 = norm(g,inf)/norm(u,inf);
    err = max(error1,error2);
    formatSpec = '%5d % 10.5e\n';
    fprintf(formatSpec,it,err)

    % prepare next iteration
    if it < maxit
      it = it + 1; u = x;
    else
      error('not converge: exceed maximum iteration count!')
    end 
  end % end of iteration
 
% plot the approximate solution
%     NP = 30;   % number of plot point for X direction
%     s =  0:pi/NP:pi;
%     app = zeros(NP+1,1);
%     for m=1:N
%       app = app + u(m)*sin(m*s');
%     end
%     plot(s,app)
%  title('hogehoge');

%
% write approximate solution to the file
%   fid = fopen('app.data','w');
%   fprintf(fid,'approximate solution\n\n');
%   fprintf(fid,'%25.16e\n',u);
%   status = fclose(fid);

% write approximate solution to the file as binary
   fid = fopen('appb.data','w');
   fwrite(fid,u,'double');
   status = fclose(fid);

   
%   B = diag(1:N);
%   W = B^3*inv(G);
%   rho = pi/2*norm(W*B);
%   rhohat = pi/2*norm(W);
%   formatSpec = 'rho = %10.5e rhohat = % 10.5e\n';
%   fprintf(formatSpec,rho,rhohat)
  toc
